@@ -66,7 +66,7 @@ class BigQueryModel(data_algebra.db_model.DBModel):
         )
         self.table_prefix = table_prefix
 
-    def quote_table_name(self, table_description):
+    def get_table_name(self, table_description):
         if not isinstance(table_description, str):
             try:
                 if table_description.node_name == "TableDescription":
@@ -81,7 +81,11 @@ class BigQueryModel(data_algebra.db_model.DBModel):
                 )
         if self.table_prefix is not None:
             table_description = self.table_prefix + "." + table_description
-        return self.quote_identifier(table_description)
+        return table_description
+
+    def quote_table_name(self, table_description):
+        table_name = self.get_table_name(table_description)
+        return self.quote_identifier(table_name)
 
     # noinspection PyMethodMayBeStatic
     def execute(self, conn, q):
@@ -127,13 +131,13 @@ class BigQueryModel(data_algebra.db_model.DBModel):
             self.drop_table(conn, table_name)
         else:
             table_exists = True
+            # noinspection PyBroadException
             try:
                 self.read_query(
                     conn,
                     "SELECT * FROM " + self.quote_table_name(table_name) + " LIMIT 1",
                 )
-                table_exists = True
-            except Exception as e:
+            except Exception:
                 table_exists = False
             if table_exists:
                 raise ValueError("table " + prepped_table_name + " already exists")
@@ -210,6 +214,7 @@ def example_handle():
     # assert os.path.isfile(credential_file)
     os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credential_file
     # os.environ["GOOGLE_APPLICATION_CREDENTIALS"]  # trigger key error if not present
+    # noinspection PyBroadException
     try:
         data_catalog = "data-algebra-test"
         data_schema = "test_1"
@@ -218,5 +223,5 @@ def example_handle():
         ).db_handle(google.cloud.bigquery.Client())
         db_handle.db_model.prepare_connection(db_handle.conn)
         return db_handle
-    except Exception as e:
+    except Exception:
         return None
