@@ -1,3 +1,7 @@
+"""
+PostgreSQL database adapter for data algebra.
+"""
+
 import data_algebra.data_ops
 import data_algebra.db_model
 
@@ -39,8 +43,7 @@ def example_handle():
     """
     # TODO: parameterize this
     assert have_sqlalchemy
-    db_handle = PostgreSQLModel().db_handle(
-        sqlalchemy.engine.create_engine(r"postgresql://johnmount@localhost/johnmount")
-    )
+    db_engine = sqlalchemy.engine.create_engine(r"postgresql://johnmount@localhost/johnmount")
+    db_handle = PostgreSQLModel().db_handle(conn=db_engine, db_engine=db_engine)
     db_handle.db_model.prepare_connection(db_handle.conn)
     return db_handle
