@@ -1,4 +1,8 @@
 
+"""
+Adapt data_algebra to SQLite database.
+"""
+
 import math
 import copy
 import numpy
@@ -20,6 +24,10 @@ from data_algebra.data_ops import *
 # at least capture to is_bad, which appears to not be implemented
 # unless we call prepare connection
 def _sqlite_is_bad_expr(dbmodel, expression):
+    """
+    Return SQL to check for bad values.
+    """
+
     return (
         "is_bad("
         + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
@@ -33,6 +41,10 @@ SQLite_formatters = {
 
 
 def _check_scalar_bad(x):
+    """
+    Return 1 if scalar value is none or nan, else 0.
+    """
+
     if x is None:
         return 1
     if not isinstance(x, numbers.Number):
@@ -43,13 +55,23 @@ def _check_scalar_bad(x):
 
 
 class MedianAgg:
+    """
+    Aggregate as median. SQLite user class.
+    """
+
     def __init__(self):
         self.collection = []
 
     def step(self, value):
+        """
+        Observe value
+        """
         self.collection.append(value)
 
     def finalize(self):
+        """
+        Return result.
+        """
         return numpy.median(self.collection)
 
 
@@ -76,6 +98,9 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         return res
 
     def prepare_connection(self, conn):
+        """
+        Insert user functions into db.
+        """
         # https://docs.python.org/3/library/sqlite3.html#sqlite3.Connection.create_function
         conn.create_function("is_bad", 1, _check_scalar_bad)
         saw = set()
@@ -205,10 +230,10 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         self, join_node, *, using=None, temp_id_source, sql_format_options=None
     ):
         assert join_node.node_name == "NaturalJoinNode"
-        assert join_node.jointype == 'RIGHT'
+        assert join_node.jointype == "RIGHT"
         # convert to left to avoid SQLite not having a right jone
         join_node_copy_right = copy.copy(join_node)
-        join_node_copy_right.jointype = 'LEFT'
+        join_node_copy_right.jointype = "LEFT"
         join_node_copy_right.sources = [join_node.sources[1], join_node.sources[0]]
         near_sql_right = data_algebra.db_model.DBModel.natural_join_to_near_sql(
             self,
@@ -225,7 +250,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
     ):
         # this is an example how to tree-rewrite the operator platform before emitting SQL.
         assert join_node.node_name == "NaturalJoinNode"
-        assert join_node.jointype == 'FULL'
+        assert join_node.jointype == "FULL"
         assert len(join_node.by) > 0  # could special case zero case later
         if temp_id_source is None:
             temp_id_source = [0]
@@ -236,36 +261,36 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         right_descr = join_node.sources[1]
         ops_simulate = (
             # get shared key set
-            left_descr
-                .project({}, group_by=join_columns)
-                .concat_rows(
-                    b=right_descr
-                        .project({}, group_by=join_columns),
-                    id_column=None,
-                )
-                .project({}, group_by=join_columns)
-                # simulate full join with left joins
-                .natural_join(
-                    b=left_descr,
-                    by=join_columns,
-                    jointype='left')
-                .natural_join(
-                    b=right_descr,
-                    by=join_columns,
-                    jointype='left')
+            left_descr.project({}, group_by=join_columns)
+            .concat_rows(
+                b=right_descr.project({}, group_by=join_columns), id_column=None,
+            )
+            .project({}, group_by=join_columns)
+            # simulate full join with left joins
+            .natural_join(b=left_descr, by=join_columns, jointype="left")
+            .natural_join(b=right_descr, by=join_columns, jointype="left")
         )
         assert isinstance(ops_simulate, NaturalJoinNode)
         simulate_near_sql = self.natural_join_to_near_sql(
             join_node=ops_simulate,
             using=using,
             temp_id_source=temp_id_source,
-            sql_format_options=sql_format_options
+            sql_format_options=sql_format_options,
         )
         return simulate_near_sql
 
     def natural_join_to_near_sql(
-        self, join_node, *, using=None, temp_id_source=None, sql_format_options=None, left_is_first=True
+        self,
+        join_node,
+        *,
+        using=None,
+        temp_id_source=None,
+        sql_format_options=None,
+        left_is_first=True
     ):
+        """
+        Translate a join into SQL, converting right and full joins to replacement code (as SQLite doesn't have these).
+        """
         if join_node.node_name != "NaturalJoinNode":
             raise TypeError(
                 "Expected join_node to be a data_algebra.data_ops.NaturalJoinNode)"
@@ -273,18 +298,20 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         assert left_is_first
         if temp_id_source is None:
             temp_id_source = [0]
-        if join_node.jointype == 'RIGHT':
+        if join_node.jointype == "RIGHT":
             return self._emit_right_join_as_left_join(
                 join_node,
                 using=using,
                 temp_id_source=temp_id_source,
-                sql_format_options=sql_format_options)
-        if join_node.jointype == 'FULL':
+                sql_format_options=sql_format_options,
+            )
+        if join_node.jointype == "FULL":
             return self._emit_full_join_as_complex(
                 join_node,
                 using=using,
                 temp_id_source=temp_id_source,
-                sql_format_options=sql_format_options)
+                sql_format_options=sql_format_options,
+            )
         # delegate back to parent class
         return data_algebra.db_model.DBModel.natural_join_to_near_sql(
             self,
