@@ -2,10 +2,6 @@
 Type defs for data operations.
 """
 
-
-from typing import List
-import collections
-
 import data_algebra.expr_rep
 import data_algebra.cdata
 import data_algebra.OrderedSet
@@ -89,6 +85,16 @@ class OperatorPlatform:
             return other.apply_to(self)
         raise TypeError("unexpected type: " + str(type(other)))
 
+    # composition
+    def add(self, other):
+        """
+        other.apply_to(self)
+
+        :param other:
+        :return:
+        """
+        return other.apply_to(self)
+
     # imitate a method
     def use(self, user_function, *args, **kwargs):
         """
@@ -101,16 +107,6 @@ class OperatorPlatform:
         """
         return user_function(self, *args, **kwargs)
 
-    # composition
-    def add(self, other):
-        """
-        other.apply_to(self)
-
-        :param other:
-        :return:
-        """
-        return other.apply_to(self)
-
     # convenience
 
     def ex(self, *, data_model=None, narrow=True, allow_limited_tables=False):
@@ -127,8 +123,11 @@ class OperatorPlatform:
     # characterization
 
     def get_tables(self):
-        """Get a dictionary of all tables used in an operator DAG,
-        raise an exception if the values are not consistent."""
+        """
+        Get a dictionary of all tables used in an operator DAG,
+        raise an exception if the values are not consistent.
+        """
+
         raise NotImplementedError("base class called")
 
     # info
@@ -188,9 +187,7 @@ class OperatorPlatform:
         if (blocks_in is None) and (blocks_out is None):
             return self  # NO-OP, return source ops
         return self.convert_records(
-            data_algebra.cdata.RecordMap(
-                blocks_in=blocks_in, blocks_out=blocks_out
-            ),
+            data_algebra.cdata.RecordMap(blocks_in=blocks_in, blocks_out=blocks_out),
         )
 
     # sklearn step style interface
