@@ -1,3 +1,6 @@
+"""
+Base class for SQL adapters for data algebra.
+"""
 
 import math
 import re
@@ -26,12 +29,18 @@ from data_algebra.OrderedSet import OrderedSet
 # Note: near sql has a bound/unbound variation treating the top layer differently than
 # subordinate nodes.
 
+
 class SQLFormatOptions(SimpleNamespace):
     """
     Simple class for holding SQL formatting options
     """
+
     def __init__(
-        self, use_with: bool = True, annotate: bool = True, sql_indent: str = " ", initial_commas: bool = False,
+        self,
+        use_with: bool = True,
+        annotate: bool = True,
+        sql_indent: str = " ",
+        initial_commas: bool = False,
     ):
         assert isinstance(use_with, bool)
         assert isinstance(annotate, bool)
@@ -48,6 +57,7 @@ class SQLFormatOptions(SimpleNamespace):
     def __str__(self):
         return self.__repr__()
 
+    # noinspection PyUnusedLocal
     def _repr_pretty_(self, p, cycle):
         """
         IPython pretty print, used at implicit print time
@@ -61,7 +71,7 @@ def _list_join_expecting_list(joiner: str, str_list: List[str]) -> List[str]:
     assert isinstance(str_list, list)
     assert all([isinstance(vi, str) for vi in str_list])
     n = len(str_list)
-    return [' ' + str_list[i] + (joiner if i < (n - 1) else '') for i in range(n)]
+    return [" " + str_list[i] + (joiner if i < (n - 1) else "") for i in range(n)]
 
 
 def _clean_annotation(annotation: Optional[str]) -> Optional[str]:
@@ -98,6 +108,7 @@ def _db_mean_expr(dbmodel, expression):
     )
 
 
+# noinspection PyUnusedLocal
 def _db_size_expr(dbmodel, expression):
     return "SUM(1)"
 
@@ -154,16 +165,24 @@ def _db_if_else_expr(dbmodel, expression):
 def _db_mapv(dbmodel, expression):
     if_expr = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
     mapping_dict = expression.args[1]
-    default_value_expr = dbmodel.expr_to_sql(expression.args[2], want_inline_parens=True)
+    default_value_expr = dbmodel.expr_to_sql(
+        expression.args[2], want_inline_parens=True
+    )
     terms = [
-        "WHEN " + dbmodel.value_to_sql(k) + " THEN " + dbmodel.value_to_sql(v) for k, v in mapping_dict.value.items()
+        "WHEN " + dbmodel.value_to_sql(k) + " THEN " + dbmodel.value_to_sql(v)
+        for k, v in mapping_dict.value.items()
     ]
     if len(terms) <= 0:
         return default_value_expr
     return (
-            "CASE" + " " + if_expr + " "
-            + " ".join(terms)
-            + " ELSE " + default_value_expr + " END"
+        "CASE"
+        + " "
+        + if_expr
+        + " "
+        + " ".join(terms)
+        + " ELSE "
+        + default_value_expr
+        + " END"
     )
 
 
@@ -293,9 +312,6 @@ def _db_nunique_expr(dbmodel, expression):
     )
 
 
-# fns that had been in bigquery_user_fns
-
-
 def _as_int64(dbmodel, expression):
     return (
         "CAST("
@@ -474,7 +490,7 @@ db_expr_formatters = {
     "**": _db_pow_expr,
     "nunique": _db_nunique_expr,
     "mapv": _db_mapv,
-    # fns that had been in bigquery_user_fns
+    # additional fns
     "as_int64": _as_int64,
     "as_str": _as_str,
     "trimstr": _trimstr,
@@ -575,10 +591,18 @@ class DBModel:
         self.union_all_term_start = union_all_term_start
         self.union_all_term_end = union_all_term_end
 
-    def db_handle(self, conn):
-        return DBHandle(db_model=self, conn=conn)
+    def db_handle(self, conn, db_engine=None):
+        """
+
+        :param conn: database connection
+        :param db_engine: optional sqlalchemy style engine (for closing)
+        """
+        return DBHandle(db_model=self, conn=conn, db_engine=db_engine)
 
     def prepare_connection(self, conn):
+        """
+        Do any augmentation or preperation of a database connection. Example: adding stored procedures.
+        """
         pass
 
     # database helpers
@@ -615,10 +639,13 @@ class DBModel:
         return r
 
     def table_exists(self, conn, table_name: str) -> bool:
+        """
+        Return true if table exists.
+        """
         assert isinstance(table_name, str)
         q_table_name = self.quote_table_name(table_name)
-        # noinspection PyBroadException
         table_exists = True
+        # noinspection PyBroadException
         try:
             self.read_query(conn, "SELECT * FROM " + q_table_name + " LIMIT 1")
         except Exception:
@@ -626,6 +653,9 @@ class DBModel:
         return table_exists
 
     def drop_table(self, conn, table_name: str, *, check: bool = True) -> None:
+        """
+        Remove a table.
+        """
         if (not check) or self.table_exists(conn, table_name):
             q_table_name = self.quote_table_name(table_name)
             self.execute(conn, self.drop_text + " " + q_table_name)
@@ -658,6 +688,9 @@ class DBModel:
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
     def read_table(self, conn, table_name: str, *, qualifiers=None, limit=None):
+        """
+        Return table contents as a Pandas data frame.
+        """
         assert isinstance(table_name, str)
         q_table_name = self.quote_table_name(table_name)
         sql = "SELECT * FROM " + q_table_name
@@ -666,6 +699,9 @@ class DBModel:
         return self.read_query(conn, sql)
 
     def read(self, conn, table):
+        """
+        Return table as a pandas data frame for table description.
+        """
         if table.node_name != "TableDescription":
             raise TypeError(
                 "Expect table to be a data_algebra.data_ops.TableDescription"
@@ -675,6 +711,9 @@ class DBModel:
         )
 
     def quote_identifier(self, identifier: str) -> str:
+        """
+        Quote identifier.
+        """
         assert isinstance(identifier, str)
         if self.identifier_quote in identifier:
             raise ValueError(
@@ -683,6 +722,9 @@ class DBModel:
         return self.identifier_quote + identifier + self.identifier_quote
 
     def quote_table_name(self, table_description) -> str:
+        """
+        Quote a table name.
+        """
         if not isinstance(table_description, str):
             try:
                 if table_description.node_name == "TableDescription":
@@ -698,6 +740,9 @@ class DBModel:
         return self.quote_identifier(table_description)
 
     def quote_string(self, string: str) -> str:
+        """
+        Quote a string value.
+        """
         assert isinstance(string, str)
         # replace all string with doubled string quotes
         return (
@@ -707,6 +752,9 @@ class DBModel:
         )
 
     def value_to_sql(self, v) -> str:
+        """
+        Convert a value to valid SQL.
+        """
         if v is None:
             return "NULL"
         if isinstance(v, data_algebra.expr_rep.ListTerm):
@@ -730,7 +778,12 @@ class DBModel:
             return "(" + ", ".join([self.value_to_sql(vi) for vi in v]) + ")"
         return str(v)
 
-    def table_values_to_sql_str_list(self, v, *, result_name: str = 'table_values') -> List[str]:
+    def table_values_to_sql_str_list(
+        self, v, *, result_name: str = "table_values"
+    ) -> List[str]:
+        """
+        Convert a table of values to a SQL. Only for small tables.
+        """
         assert v is not None
         m = v.shape[0]
         assert m > 0
@@ -740,6 +793,7 @@ class DBModel:
         qv = self.value_to_sql
 
         def q_row(i):
+            """quote row"""
             return (
                 self.union_all_term_start
                 + "SELECT "
@@ -755,11 +809,14 @@ class DBModel:
         sql = (
             ["SELECT", " *", "FROM ("]
             + ["    " + ("" if (i < 1) else "UNION ALL ") + q_row(i) for i in range(m)]
-            + [f') {qi(result_name)}']
+            + [f") {qi(result_name)}"]
         )
         return sql
 
     def expr_to_sql(self, expression, *, want_inline_parens: bool = False) -> str:
+        """
+        Convert an expression to SQL.
+        """
         if isinstance(expression, str):
             return expression
         assert isinstance(expression, data_algebra.expr_rep.PreTerm)
@@ -797,7 +854,9 @@ class DBModel:
             return self.value_to_sql(expression.value)
         raise TypeError("unexpected type: " + str(type(expression)))
 
-    def _indent_and_sep_terms(self, terms, *, sep: str = ",", sql_format_options=None) -> List[str]:
+    def _indent_and_sep_terms(
+        self, terms, *, sep: str = ",", sql_format_options=None
+    ) -> List[str]:
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         n = len(terms)
@@ -821,6 +880,9 @@ class DBModel:
     def table_def_to_near_sql(
         self, table_def, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert a table description to NearSQL.
+        """
         if table_def.node_name != "TableDescription":
             raise TypeError(
                 "Expected table_def to be a data_algebra.data_ops.TableDescription)"
@@ -836,7 +898,7 @@ class DBModel:
 
         subsql = data_algebra.near_sql.NearSQLTable(
             terms={k: self.quote_identifier(k) for k in cols_using},
-            quoted_query_name=self.quote_table_name(table_def),
+            table_name=self.quote_table_name(table_def),
             quoted_table_name=self.quote_table_name(table_def),
         )
         near_sql = subsql
@@ -851,6 +913,7 @@ class DBModel:
             temp_id_source[0] = temp_id_source[0] + 1
             near_sql = data_algebra.near_sql.NearSQLUnaryStep(
                 terms=terms,
+                query_name=view_name,
                 quoted_query_name=self.quote_identifier(view_name),
                 sub_sql=subsql.to_bound_near_sql(columns=using),
             )
@@ -859,6 +922,9 @@ class DBModel:
     def extend_to_near_sql(
         self, extend_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert an extend step into NearSQL.
+        """
         if extend_node.node_name != "ExtendNode":
             raise TypeError(
                 "Expected extend_node to be a data_algebra.data_ops.ExtendNode)"
@@ -922,9 +988,9 @@ class DBModel:
             oi.get_column_names(cols_used_in_term)
             cols_used_in_term.update(window_vars)
             declared_term_dependencies[ci] = cols_used_in_term
-        annotation = str(extend_node.to_python_implementation(
-            print_sources=False, indent=-1
-        ))
+        annotation = str(
+            extend_node.to_python_implementation(print_sources=False, indent=-1)
+        )
         # TODO: see if we can merge with subsql instead of building a new one
         if (
             self.allow_extend_merges
@@ -935,6 +1001,7 @@ class DBModel:
         ):
             # check detailed merge conditions
             def non_trivial_terms(*, dep_dict, term_dict):
+                """pickout non-trivial terms"""
                 return [
                     ki
                     for ki, vi in dep_dict.items()
@@ -982,6 +1049,7 @@ class DBModel:
         temp_id_source[0] = temp_id_source[0] + 1
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             annotation=annotation,
@@ -993,6 +1061,9 @@ class DBModel:
     def project_to_near_sql(
         self, project_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert a project step to NearSQL
+        """
         if project_node.node_name != "ProjectNode":
             raise TypeError(
                 "Expected project_node to be a data_algebra.data_ops.ProjectNode)"
@@ -1018,12 +1089,13 @@ class DBModel:
             )
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
-            annotation=str(project_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                project_node.to_python_implementation(print_sources=False, indent=-1)
+            ),
         )
         return near_sql
 
@@ -1035,6 +1107,9 @@ class DBModel:
         temp_id_source=None,
         sql_format_options=None,
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert select rows into NearSQL
+        """
         if select_rows_node.node_name != "SelectRowsNode":
             raise TypeError(
                 "Expected select_rows_node to be a data_algebra.data_ops.SelectRowsNode)"
@@ -1058,12 +1133,15 @@ class DBModel:
         ]
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
-            annotation=str(select_rows_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                select_rows_node.to_python_implementation(
+                    print_sources=False, indent=-1
+                )
+            ),
         )
         return near_sql
 
@@ -1075,6 +1153,9 @@ class DBModel:
         temp_id_source=None,
         sql_format_options=None,
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert select columns to NearSQL.
+        """
         if select_columns_node.node_name != "SelectColumnsNode":
             raise TypeError(
                 "Expected select_columns_to_near_sql to be a data_algebra.data_ops.SelectColumnsNode)"
@@ -1109,6 +1190,9 @@ class DBModel:
         temp_id_source=None,
         sql_format_options=None,
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert drop columns to NearSQL
+        """
         if drop_columns_node.node_name != "DropColumnsNode":
             raise TypeError(
                 "Expected drop_columns_node to be a data_algebra.data_ops.DropColumnsNode)"
@@ -1132,6 +1216,9 @@ class DBModel:
     def order_to_near_sql(
         self, order_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert order rows to NearSQL.
+        """
         if order_node.node_name != "OrderRowsNode":
             raise TypeError(
                 "Expected order_node to be a data_algebra.data_ops.OrderRowsNode)"
@@ -1170,18 +1257,22 @@ class DBModel:
             suffix = suffix + ["LIMIT " + order_node.limit.__repr__()]
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
-            annotation=str(order_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                order_node.to_python_implementation(print_sources=False, indent=-1)
+            ),
         )
         return near_sql
 
     def rename_to_near_sql(
         self, rename_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert rename columns to NearSQL.
+        """
         if rename_node.node_name != "RenameColumnsNode":
             raise TypeError(
                 "Expected rename_node to be a data_algebra.data_ops.RenameColumnsNode)"
@@ -1206,11 +1297,12 @@ class DBModel:
         terms.update({vi: None for vi in unchanged_columns})
         near_sql = data_algebra.near_sql.NearSQLUnaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
-            annotation=str(rename_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                rename_node.to_python_implementation(print_sources=False, indent=-1)
+            ),
         )
         return near_sql
 
@@ -1218,6 +1310,9 @@ class DBModel:
         coalesce_formatter = self.sql_formatters["coalesce"]
 
         class PseudoExpression:
+            """
+            Class to carry info into a method expecting an actual expression type.
+            """
             def __init__(self, args):
                 self.args = args.copy()
 
@@ -1235,9 +1330,7 @@ class DBModel:
         }
         return terms
 
-    def _natural_join_sub_queries(
-            self,  *, join_node, using, temp_id_source
-    ):
+    def _natural_join_sub_queries(self, *, join_node, using, temp_id_source):
         if join_node.node_name != "NaturalJoinNode":
             raise TypeError(
                 "Expected join_node to be a data_algebra.data_ops.NaturalJoinNode)"
@@ -1250,7 +1343,9 @@ class DBModel:
         missing = using - set(join_node.column_names)
         if len(missing) > 0:
             raise KeyError("referred to unknown columns: " + str(missing))
-        using_left, using_right = join_node.columns_used_from_sources(using=using.union(by_set))
+        using_left, using_right = join_node.columns_used_from_sources(
+            using=using.union(by_set)
+        )
         sql_left = join_node.sources[0].to_near_sql_implementation(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
@@ -1260,8 +1355,17 @@ class DBModel:
         return using_left, sql_left, using_right, sql_right
 
     def natural_join_to_near_sql(
-        self, join_node, *, using=None, temp_id_source=None, sql_format_options=None, left_is_first=True
+        self,
+        join_node,
+        *,
+        using=None,
+        temp_id_source=None,
+        sql_format_options=None,
+        left_is_first=True,
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert natural join into NearSQL.
+        """
         if temp_id_source is None:
             temp_id_source = [0]
         if using is None:
@@ -1306,23 +1410,27 @@ class DBModel:
                 on_terms = on_terms + [self.on_end]
         near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql1=sql_left.to_bound_near_sql(columns=using_left, force_sql=False),
             joiner=join_node.jointype + " JOIN",
             sub_sql2=sql_right.to_bound_near_sql(columns=using_right, force_sql=False),
             suffix=on_terms,
-            annotation=str(join_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                join_node.to_python_implementation(print_sources=False, indent=-1)
+            ),
         )
         return near_sql
 
     def concat_rows_to_near_sql(
         self, concat_node, *, using=None, temp_id_source=None, sql_format_options=None
     ) -> data_algebra.near_sql.NearSQL:
+        """
+        Convert concat rows into NearSQL.
+        """
         if concat_node.node_name != "ConcatRowsNode":
             raise TypeError(
-                "Expected join_node to be a data_algebra.data_ops.ConcatRowsNode)"
+                "Expected concat_node to be a data_algebra.data_ops.ConcatRowsNode)"
             )
         if temp_id_source is None:
             temp_id_source = [0]
@@ -1359,6 +1467,7 @@ class DBModel:
             terms.update({concat_node.id_column: None})
         near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
+            query_name=view_name,
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql1=sql_left.to_bound_near_sql(
                 columns=using_left, force_sql=True, constants=constants_left,
@@ -1367,15 +1476,16 @@ class DBModel:
             sub_sql2=sql_right.to_bound_near_sql(
                 columns=using_left, force_sql=True, constants=constants_right,
             ),
-            annotation=str(concat_node.to_python_implementation(
-                print_sources=False, indent=-1
-            )),
+            annotation=str(
+                concat_node.to_python_implementation(print_sources=False, indent=-1)
+            ),
         )
         return near_sql
 
-    def to_sql(
-        self, ops, *, sql_format_options=None,
-    ) -> str:
+    def to_sql(self, ops, *, sql_format_options=None,) -> str:
+        """
+        Convert ViewRepresentation into SQL string.
+        """
         assert isinstance(self, DBModel)
         assert isinstance(ops, data_algebra.data_ops.ViewRepresentation)
         if sql_format_options is None:
@@ -1438,7 +1548,12 @@ class DBModel:
         sql_str_list = [v for v in sql_str_list if len(v) > 0]
         return "\n".join(sql_str_list) + "\n"
 
-    def row_recs_to_blocks_query_str_list_pair(self, record_spec) -> Tuple[List[str], List[str]]:
+    def row_recs_to_blocks_query_str_list_pair(
+        self, record_spec
+    ) -> Tuple[List[str], List[str]]:
+        """
+        Convert row recs to blocks transformation into structures to help with SQL conversion.
+        """
         control_value_cols = [
             c
             for c in record_spec.control_table.columns
@@ -1488,10 +1603,9 @@ class DBModel:
             cstmt = cstmt + " ELSE NULL END AS " + self.quote_identifier(result_col)
             col_stmts.append(cstmt)
         ctab_sql = self.table_values_to_sql_str_list(record_spec.control_table)
-        sql_prefix = (
-            _list_join_expecting_list(",", col_stmts)
-            + ["FROM ( SELECT * FROM "]
-        )
+        sql_prefix = _list_join_expecting_list(",", col_stmts) + [
+            "FROM ( SELECT * FROM "
+        ]
         sql_suffix = (
             [" ) a"]
             + ["CROSS JOIN ("]
@@ -1503,7 +1617,12 @@ class DBModel:
         return sql_prefix, sql_suffix
 
     # noinspection PyUnusedLocal
-    def blocks_to_row_recs_query_str_list_pair(self, record_spec) -> Tuple[List[str], List[str]]:
+    def blocks_to_row_recs_query_str_list_pair(
+        self, record_spec
+    ) -> Tuple[List[str], List[str]]:
+        """
+        Convert blocks to row recs transform into structures to help with SQL translation.
+        """
         assert record_spec.control_table.shape[0] >= 1
         col_stmts = []
         for c in record_spec.record_keys:
@@ -1516,15 +1635,15 @@ class DBModel:
                 if cc not in record_spec.control_table_keys:
                     cc0 = record_spec.control_table[cc][0]
                     col_stmts.append(
-                        " " + self.quote_identifier(cc) + " AS " + self.quote_identifier(cc0)
+                        " "
+                        + self.quote_identifier(cc)
+                        + " AS "
+                        + self.quote_identifier(cc0)
                     )
-            sql_prefix = (
-                    _list_join_expecting_list(',', col_stmts)
-                    + ["FROM ( SELECT * FROM "]
-            )
-            sql_suffix = (
-                [" ) a"]
-            )
+            sql_prefix = _list_join_expecting_list(",", col_stmts) + [
+                "FROM ( SELECT * FROM "
+            ]
+            sql_suffix = [" ) a"]
             return sql_prefix, sql_suffix
         control_value_cols = [
             c
@@ -1559,21 +1678,23 @@ class DBModel:
                         + self.quote_identifier(col[i])
                     )
                     col_stmts.append(cstmt)
-        sql_prefix = (
-                _list_join_expecting_list(',', col_stmts)
-                + ["FROM ( SELECT * FROM "]
-        )
+        sql_prefix = _list_join_expecting_list(",", col_stmts) + [
+            "FROM ( SELECT * FROM "
+        ]
         sql_suffix = (
-                [" ) a"]
-                + ["GROUP BY"]
-                + _list_join_expecting_list(',', control_cols)
-                + ["ORDER BY "]  # order by not required, but nice to have
-                + _list_join_expecting_list(',', control_cols)
+            [" ) a"]
+            + ["GROUP BY"]
+            + _list_join_expecting_list(",", control_cols)
+            + ["ORDER BY "]  # order by not required, but nice to have
+            + _list_join_expecting_list(",", control_cols)
         )
         return sql_prefix, sql_suffix
 
     # encode and name a term for use in a SQL expression
     def enc_term_(self, k, *, terms) -> str:
+        """
+        encode and name a term for use in a SQL expression
+        """
         v = None
         try:
             v = terms[k]
@@ -1586,11 +1707,16 @@ class DBModel:
     def convert_nearsql_container_subsql_(
         self, nearsql_container, *, sql_format_options=None
     ) -> List[str]:
+        """
+        Convert sub-SQL into list of SQL string lines.
+        """
         assert isinstance(nearsql_container, data_algebra.near_sql.NearSQLContainer)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         assert isinstance(sql_format_options, SQLFormatOptions)
-        sub_sql = nearsql_container.to_sql_str_list(self, sql_format_options=sql_format_options)
+        sub_sql = nearsql_container.to_sql_str_list(
+            self, sql_format_options=sql_format_options
+        )
         assert isinstance(sub_sql, list)
         if isinstance(nearsql_container.near_sql, data_algebra.near_sql.NearSQLTable):
             sql = sub_sql
@@ -1619,6 +1745,9 @@ class DBModel:
         constants=None,
         sql_format_options=None,
     ) -> List[str]:
+        """
+        Convert SQL common table expression to list of SQL string lines.
+        """
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLCommonTableExpression)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
@@ -1641,6 +1770,9 @@ class DBModel:
         constants=None,
         sql_format_options=None,
     ) -> List[str]:
+        """
+        Convert SQL table description to list of SQL string lines.
+        """
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLTable)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
@@ -1661,9 +1793,7 @@ class DBModel:
                     for (k, v) in constants.items()
                 ]
             if len(terms_strs) < 1:
-                terms_strs = [
-                    '*'
-                ]
+                terms_strs = ["*"]
             return (
                 ["SELECT"]
                 + self._indent_and_sep_terms(
@@ -1683,6 +1813,9 @@ class DBModel:
         constants=None,
         sql_format_options=None,
     ) -> List[str]:
+        """
+        Convert SQL unary operation to list of SQL string lines.
+        """
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLUnaryStep)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
@@ -1699,9 +1832,7 @@ class DBModel:
                     for (k, v) in constants.items()
                 ]
             if len(terms_strs) < 1:
-                terms_strs = [
-                    '*'
-                ]
+                terms_strs = ["*"]
         sql_start = "SELECT"
         if (
             sql_format_options.annotate
@@ -1727,12 +1858,11 @@ class DBModel:
         return sql
 
     def nearsqlrawq_to_sql_str_list_(
-        self,
-        near_sql,
-        *,
-        sql_format_options=None,
-        add_select=True,
+        self, near_sql, *, sql_format_options=None, add_select=True,
     ) -> List[str]:
+        """
+        Convert user SQL query to list of SQL string lines.
+        """
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLRawQStep)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
@@ -1742,7 +1872,7 @@ class DBModel:
             sql = sql + ["-- " + _clean_annotation(near_sql.annotation)]
         if add_select:
             sql = sql + ["SELECT"]
-        sql = sql + [' ' + v for v in near_sql.prefix]
+        sql = sql + [" " + v for v in near_sql.prefix]
         if near_sql.sub_sql is not None:
             sql = sql + [
                 sql_format_options.sql_indent + si
@@ -1751,7 +1881,7 @@ class DBModel:
                 )
             ]
         if (near_sql.suffix is not None) and (len(near_sql.suffix) > 0):
-            sql = sql + [' ' + v for v in near_sql.suffix]
+            sql = sql + [" " + v for v in near_sql.suffix]
         return sql
 
     def nearsqlbinary_to_sql_str_list_(
@@ -1764,6 +1894,9 @@ class DBModel:
         sql_format_options=None,
         quoted_query_name=None,
     ) -> List[str]:
+        """
+        Convert SQL binary operation to list of SQL string lines.
+        """
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLBinaryStep)
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
@@ -1775,9 +1908,7 @@ class DBModel:
             terms.update(constants)
         terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
         if len(terms_strs) < 1:
-            terms_strs = [
-                '*'
-            ]
+            terms_strs = ["*"]
         is_union = "union" in near_sql.joiner.lower()
         sql_start = "SELECT"
         if (
@@ -1831,9 +1962,19 @@ class DBModel:
 
 
 class DBHandle:
-    def __init__(self, *, db_model: DBModel, conn):
+    """
+    Container for database connection handles.
+    """
+    def __init__(self, *, db_model: DBModel, conn, db_engine=None):
+        """
+
+        :param db_model: associated database model
+        :param conn: database connection
+        :param db_engine: optional sqlalchemy style engine (for closing)
+        """
         assert isinstance(db_model, DBModel)
         self.db_model = db_model
+        self.db_engine = db_engine
         self.conn = conn
 
     def __enter__(self):
@@ -1844,9 +1985,17 @@ class DBHandle:
         self.close()
 
     def read_query(self, q):
+        """
+        Return results of query as a Pandas table.
+        """
         return self.db_model.read_query(conn=self.conn, q=q)
 
-    def describe_table(self, table_name: str, *, qualifiers=None, row_limit: Optional[int] = 7):
+    def describe_table(
+        self, table_name: str, *, qualifiers=None, row_limit: Optional[int] = 7
+    ):
+        """
+        Return a description of a database table.
+        """
         head = self.read_query(
             q="SELECT * FROM "
             + self.db_model.quote_table_name(table_name)
@@ -1858,27 +2007,45 @@ class DBHandle:
         )
 
     def execute(self, q) -> None:
+        """
+        Execute a SQL query.
+        """
         self.db_model.execute(conn=self.conn, q=q)
 
     def drop_table(self, table_name: str) -> None:
+        """
+        Remove a table.
+        """
         self.db_model.drop_table(self.conn, table_name)
 
     def insert_table(self, d, *, table_name: str, allow_overwrite: bool = False):
+        """
+        Insert a table into the database.
+        """
         self.db_model.insert_table(
             conn=self.conn, d=d, table_name=table_name, allow_overwrite=allow_overwrite
         )
         return self.describe_table(table_name)
 
-    def to_sql(
-        self, ops, *, sql_format_options=None,
-    ) -> str:
+    def to_sql(self, ops, *, sql_format_options=None,) -> str:
+        """
+        Convert operations into SQL
+        """
         return self.db_model.to_sql(ops=ops, sql_format_options=sql_format_options,)
 
     def query_to_csv(self, q, *, res_name: str) -> None:
+        """
+        Execute a query and save the results as a CSV file.
+        """
         d = self.read_query(q)
         d.to_csv(res_name, index=False)
 
-    def table_values_to_sql_str_list(self, v, *, result_name: str = 'table_values') -> List[str]:
+    def table_values_to_sql_str_list(
+        self, v, *, result_name: str = "table_values"
+    ) -> List[str]:
+        """
+        Convert a table of values to a SQL. Only for small tables.
+        """
         return self.db_model.table_values_to_sql_str_list(v, result_name=result_name)
 
     def __str__(self):
@@ -1896,9 +2063,25 @@ class DBHandle:
         return self.__str__()
 
     def close(self) -> None:
+        """
+        Dispose of engine, or close connection.
+        """
         if self.conn is not None:
-            try:
-                self.conn.close()
-            except Exception:
-                pass
+            caught = None
+            if self.db_engine is not None:
+                # sqlalchemy style handle
+                # noinspection PyBroadException
+                try:
+                    self.db_engine.dispose()
+                except Exception as ex:
+                    caught = ex
+            else:
+                # noinspection PyBroadException
+                try:
+                    self.conn.close()
+                except Exception as ex:
+                    caught = ex
+            self.db_engine = None
             self.conn = None
+            if caught is not None:
+                raise ValueError('close caught: ' + str(caught))
