@@ -1,3 +1,7 @@
+"""
+Represent data processing expressions.
+"""
+
 from abc import ABC
 from typing import Optional, Union
 
@@ -19,6 +23,7 @@ class PythonText:
     Class for holding text representation of Python, with possible additional annotations.
     str() method returns only the text for interoperability.
     """
+
     def __init__(self, s: str, *, is_in_parens: bool = False):
         assert isinstance(s, str)
         assert isinstance(is_in_parens, bool)
@@ -117,6 +122,9 @@ class PreTerm(ABC):
         self.source_string = None
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         raise NotImplementedError("base method called")
 
@@ -129,6 +137,9 @@ class PreTerm(ABC):
         raise NotImplementedError("base class called")
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         raise NotImplementedError("base class called")
 
     # analysis
@@ -144,6 +155,9 @@ class PreTerm(ABC):
     # eval
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         raise NotImplementedError("base class called")
 
     # emitters
@@ -152,13 +166,21 @@ class PreTerm(ABC):
         """
         Convert parsed expression into a string
 
-        :param want_inline_parens: bool if True put parens around complex expressions that don't already have a grouper.
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
         """
         raise NotImplementedError(
             "base class method called"
         )  # https://docs.python.org/3/library/exceptions.html
 
-    def to_source(self, *, want_inline_parens=False, dialect="Python"):
+    def to_source(self, *, want_inline_parens=False, dialect="Python") -> PythonText:
+        """
+        Convert to source code.
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :param dialect: dialect to emit (not currently used)
+        :return: PythonText
+        """
         if dialect == "Python":
             return self.to_python(want_inline_parens=want_inline_parens)
         else:
@@ -187,6 +209,9 @@ def _check_expr_incompatible_types(a, b):
     """
 
     def obvious_declared_type(v):
+        """
+        Return type of value, if obvious (usually it is not).
+        """
         if v is None:
             return None  # None may be a placeholder for "don't know at this point"
         if isinstance(v, Value):
@@ -233,7 +258,8 @@ class Term(PreTerm, ABC):
                 raise TypeError(
                     f"trying to combine incompatible values:"
                     + f" {self}:{obvious_type_problem[0]}"
-                    + f" and {other}:{obvious_type_problem[0]} with {op}")
+                    + f" and {other}:{obvious_type_problem[0]} with {op}"
+                )
         return Expression(op, (self, other), inline=inline, method=method)
 
     def __rop_expr__(self, op, other):
@@ -374,103 +400,202 @@ class Term(PreTerm, ABC):
     # TODO: double check https://docs.python.org/3/library/operator.html
 
     def sign(self):
+        """
+        Return -1, 0, 1 as sign of item (vectorized).
+        """
         return self.__uop_expr__("sign")
 
     def sin(self):
+        """
+        Return trigometric sin() (in radians) of item (vectorized).
+        """
         return self.__uop_expr__("sin")
 
     def cos(self):
+        """
+        Return trigometric cos() (in radians) of item (vectorized).
+        """
         return self.__uop_expr__("cos")
 
     def arcsin(self):
+        """
+        Return trigometric arcsin() (in radians) of item (vectorized).
+        """
         return self.__uop_expr__("arcsin")
 
     def arccos(self):
+        """
+        Return trigometric arccos() (in radians) of item (vectorized).
+        """
         return self.__uop_expr__("arccos")
 
     def arctan(self):
+        """
+        Return trigometric arctan() (in radians) of item (vectorized).
+        """
         return self.__uop_expr__("arctan")
 
     def arctan2(self, other):
+        """
+        Return trigometric arctan2() (in radians) of item (vectorized).
+        """
         return self.__op_expr__("arctan2", other)
 
     def sinh(self):
+        """
+        Return hyperbolic sinh() of item (vectorized).
+        """
         return self.__uop_expr__("sinh")
 
     def cosh(self):
+        """
+        Return hyperbolic cosh() of item (vectorized).
+        """
         return self.__uop_expr__("cosh")
 
     def tanh(self):
+        """
+        Return hyperbolic tanh() of item (vectorized).
+        """
         return self.__uop_expr__("tanh")
 
     def arcsinh(self):
+        """
+        Return hyperbolic arcsinh() of item (vectorized).
+        """
         return self.__uop_expr__("arcsinh")
 
     def arccosh(self):
+        """
+        Return hyperbolic arccosh() of item (vectorized).
+        """
         return self.__uop_expr__("arccosh")
 
     def arctanh(self):
+        """
+        Return hyperbolic arctanh() of item (vectorized).
+        """
         return self.__uop_expr__("arctanh")
 
     def floor(self):
+        """
+        Return floor() (largest int no larger than, as real type) of item (vectorized).
+        """
         return self.__uop_expr__("floor")
 
     def ceil(self):
+        """
+        Return ceil() (smallest int no smaller than, as real type) of item (vectorized).
+        """
         return self.__uop_expr__("ceil")
 
     def sum(self):
+        """
+        Return sum() of items (vectorized).
+        """
         return self.__uop_expr__("sum")
 
     def cumprod(self):
+        """
+        Return cumprod() of items (vectorized).
+        """
         return self.__uop_expr__("cumprod")
 
     def cumsum(self):
+        """
+        Return cumsum() of items (vectorized).
+        """
         return self.__uop_expr__("cumsum")
 
     def exp(self):
+        """
+        Return exp() of items (vectorized).
+        """
         return self.__uop_expr__("exp")
 
     def expm1(self):
+        """
+        Return exp() - 1 of items (vectorized).
+        """
         return self.__uop_expr__("expm1")
 
     def log(self):
+        """
+        Return base e logarithm of items (vectorized).
+        """
         return self.__uop_expr__("log")
 
     def log10(self):
+        """
+        Return base 10 logarithm of items (vectorized).
+        """
         return self.__uop_expr__("log10")
 
     def log1p(self):
+        """
+        Return base e logarithm of 1 + items (vectorized).
+        """
         return self.__uop_expr__("log1p")
 
     def mod(self, other):
+        """
+        Return modulo of items (vectorized).
+        """
         return self.__op_expr__("mod", other)
 
     def remainder(self, other):
+        """
+        Return remainder of items (vectorized).
+        """
         return self.__op_expr__("remainder", other, inline=False, method=True)
 
     def sqrt(self):
+        """
+        Return sqrt of items (vectorized).
+        """
         return self.__uop_expr__("sqrt")
 
     def abs(self):
+        """
+        Return absolute value of items (vectorized).
+        """
         return self.__uop_expr__("abs")
 
     def maximum(self, other):
+        """
+        Return per row maximum of items and other (vectorized).
+        """
         return self.__op_expr__("maximum", other, method=True, inline=False)
 
     def minimum(self, other):
+        """
+        Return per row minimum of items and other (vectorized).
+        """
         return self.__op_expr__("minimum", other, method=True, inline=False)
 
     def fmax(self, other):
+        """
+        Return per row fmax of items and other (vectorized).
+        """
         return self.__op_expr__("fmax", other, inline=False)
 
     def fmin(self, other):
+        """
+        Return per row fmin of items and other (vectorized).
+        """
         return self.__op_expr__("fmin", other, inline=False)
 
     # more numpy stuff
     def round(self):
+        """
+        Return rounded values (nearest integer, subject to some rules) as real (vectorized).
+        """
         return self.__uop_expr__("round")
 
     def around(self, other):
+        """
+        Return rounded values (given numer of decimals) as real (vectorized).
+        """
         assert isinstance(other, Value)  # digits control
         return self.__op_expr__("around", other, inline=False)
 
@@ -478,92 +603,137 @@ class Term(PreTerm, ABC):
     # https://pandas.pydata.org/pandas-docs/stable/reference/groupby.html
 
     def all(self):
+        """
+        Return True if all items True (vectorized).
+        """
         return self.__uop_expr__("all")
 
     def any(self):
+        """
+        Return True if any items True (vectorized).
+        """
         return self.__uop_expr__("any")
 
     def bfill(self):
+        """
+        Return vector with missing vallues filled (vectorized).
+        """
         return self.__uop_expr__("bfill")
 
     def count(self):
+        """
+        Return number of non-NA cells (vectorized).
+        """
         return self.__uop_expr__("count")
 
     def cumcount(self):
+        """
+        Return cumulative number of non-NA cells (vectorized).
+        """
         return self.__uop_expr__("cumcount")
 
     def cummax(self):
+        """
+        Return cumulative maximum (vectorized).
+        """
         return self.__uop_expr__("cummax")
 
     def cummin(self):
+        """
+        Return cumulative minimum (vectorized).
+        """
         return self.__uop_expr__("cummin")
 
     def ffill(self):
+        """
+        Return vector with missing vallues filled (vectorized).
+        """
         return self.__uop_expr__("ffill")
 
-    def first(self):
-        return self.__uop_expr__("first")
-
-    def head(self):
-        return self.__uop_expr__("head")
-
     def is_monotonic_decreasing(self):
+        """
+        Return vector True if monotonic decreasing (vectorized).
+        """
         return self.__uop_expr__("is_monotonic_decreasing")
 
     def is_monotonic_increasing(self):
+        """
+        Return vector True if monotonic increasing (vectorized).
+        """
         return self.__uop_expr__("is_monotonic_increasing")
 
     def last(self):
+        """
+        Return last (vectorized).
+        """
         return self.__uop_expr__("last")
 
     def max(self):
+        """
+        Return last (vectorized).
+        """
         return self.__uop_expr__("max")
 
     def mean(self):
+        """
+        Return mean (vectorized).
+        """
         return self.__uop_expr__("mean")
 
     def median(self):
+        """
+        Return median (vectorized).
+        """
         return self.__uop_expr__("median")
 
     def min(self):
+        """
+        Return min (vectorized).
+        """
         return self.__uop_expr__("min")
 
     def ngroup(self):
+        """
+        Return number of groups (vectorized).
+        """
         return self.__uop_expr__("ngroup")
 
-    def nlargest(self):
-        return self.__uop_expr__("nlargest")
-
-    def nsmallest(self):
-        return self.__uop_expr__("nsmallest")
-
-    def nth(self):
-        return self.__uop_expr__("nth")
-
     def nunique(self):
+        """
+        Return number of unique items (vectorized).
+        """
         return self.__uop_expr__("nunique")
 
     def rank(self):
+        """
+        Return item rangings (vectorized).
+        """
         return self.__uop_expr__("rank")
 
     def size(self):
+        """
+        Return number of items (vectorized).
+        """
         return self.__uop_expr__("size")
 
     def std(self):
+        """
+        Return standard devaition (vectorized).
+        """
         return self.__uop_expr__("std")
 
-    def unique(self):
-        return self.__uop_expr__("unique")
-
-    def value_counts(self):
-        return self.__uop_expr__("value_counts")
-
     def var(self):
+        """
+        Return variance (vectorized).
+        """
         return self.__uop_expr__("var")
 
     # pandas shift
 
     def shift(self, periods=None):
+        """
+        Return shifted items (vectorized).
+        """
         if periods is None:
             periods = Value(1)
         assert isinstance(periods, Value)  # digits control
@@ -572,49 +742,82 @@ class Term(PreTerm, ABC):
     # our ad-hoc definitions
 
     def is_null(self):
+        """
+        Return which items are null (vectorized).
+        """
         return self.__uop_expr__("is_null")
 
     def is_bad(self):
+        """
+        Return which items are bad (null or nan) (vectorized).
+        """
         return self.__uop_expr__("is_bad")
 
     def if_else(self, x, y):
+        """
+        Vectorized selection between two argument vectors.
+        """
         # could check if x and y are compatible types
         return self.__triop_expr__("if_else", x, y, method=True)
 
     def is_in(self, x):
-        return self.__op_expr__("is_in", x, inline=False, method=True, check_types=False,)
+        """
+        Set membership (vectorized).
+        """
+        return self.__op_expr__(
+            "is_in", x, inline=False, method=True, check_types=False,
+        )
 
     def concat(self, x):
+        """
+        Concatinate strings (vectorized).
+        """
         # TODO: see if we can format back to infix notation
-        return self.__op_expr__("concat", x, inline=False, method=True, check_types=False,)
+        return self.__op_expr__(
+            "concat", x, inline=False, method=True, check_types=False,
+        )
 
     def coalesce(self, x):
+        """
+        Replace missing values with alternative (vectorized).
+        """
         # TODO: see if we can format back to infix notation
         return self.__op_expr__("coalesce", x, inline=False, method=True)
 
     def co_equalizer(self, x):
+        """
+        Compute the connected components (co-equalizer).
+        """
         return self.__op_expr__("co_equalizer", x, inline=False, method=True)
 
     def mapv(self, value_map, default_value):
         """
-        Map values.
+        Map values (vectorized).
         """
-
         assert isinstance(value_map, DictTerm)
         assert isinstance(default_value, Value)
         return self.__triop_expr__(
             "mapv", x=value_map, y=default_value, inline=False, method=True
         )
 
-    # fns that had been in bigquery_user_fns
+    # additional fns
 
     def as_int64(self):
+        """
+        Cast as int (vectorized).
+        """
         return self.__uop_expr__("as_int64")
 
     def as_str(self):
+        """
+        Cast as string (vectorized).
+        """
         return self.__uop_expr__("as_str")
 
     def trimstr(self, start, stop):
+        """
+        Trim string start (inclusive) to stop (exclusive) (vectorized).
+        """
         assert isinstance(start, Value)
         assert isinstance(stop, Value)
         return self.__triop_expr__(
@@ -622,66 +825,128 @@ class Term(PreTerm, ABC):
         )
 
     def coalesce_0(self):
+        """
+        Replace missing values with zero (vectorized).
+        """
         return self.coalesce(Value(0))
 
     def datetime_to_date(self):
+        """
+        Convert date time to date (vectorized).
+        """
         return self.__uop_expr__("datetime_to_date")
 
     def parse_datetime(self, format=None):
+        """
+        Parse string as a date time (vectorized).
+        """
         if format is None:
             format = Value("%Y-%m-%d %H:%M:%S")
         assert isinstance(format, Value)
         return self.__op_expr__(
-            "parse_datetime", other=format, inline=False, method=True, check_types=False,
+            "parse_datetime",
+            other=format,
+            inline=False,
+            method=True,
+            check_types=False,
         )
 
     def parse_date(self, format=None):
+        """
+        Parse string as a date (vectorized).
+        """
         if format is None:
             format = Value("%Y-%m-%d")
-        return self.__op_expr__("parse_date", other=format, inline=False, method=True, check_types=False,)
+        return self.__op_expr__(
+            "parse_date", other=format, inline=False, method=True, check_types=False,
+        )
 
     def format_datetime(self, format=None):
+        """
+        Format string as a date time (vectorized).
+        """
         if format is None:
             format = Value("%Y-%m-%d %H:%M:%S")
         assert isinstance(format, Value)
         return self.__op_expr__(
-            "format_datetime", other=format, inline=False, method=True, check_types=False,
+            "format_datetime",
+            other=format,
+            inline=False,
+            method=True,
+            check_types=False,
         )
 
     def format_date(self, format=None):
+        """
+        Format string as a date (vectorized).
+        """
         if format is None:
             format = Value("%Y-%m-%d")
-        return self.__op_expr__("format_date", other=format, inline=False, method=True, check_types=False,)
+        return self.__op_expr__(
+            "format_date", other=format, inline=False, method=True, check_types=False,
+        )
 
     def dayofweek(self):
+        """
+        Convert date to date of week (vectorized).
+        """
         return self.__uop_expr__("dayofweek")
 
     def dayofyear(self):
+        """
+        Convert date to date of year (vectorized).
+        """
         return self.__uop_expr__("dayofyear")
 
     def dayofmonth(self):
+        """
+        Convert date to day of month (vectorized).
+        """
         return self.__uop_expr__("dayofmonth")
 
     def weekofyear(self):
+        """
+        Convert date to week of year (vectorized).
+        """
         return self.__uop_expr__("weekofyear")
 
     def month(self):
+        """
+        Convert date to month (vectorized).
+        """
         return self.__uop_expr__("month")
 
     def quarter(self):
+        """
+        Convert date to quarter (vectorized).
+        """
         return self.__uop_expr__("quarter")
 
     def year(self):
+        """
+        Convert date to year (vectorized).
+        """
         return self.__uop_expr__("year")
 
     def timestamp_diff(self, other):
+        """
+        Compute difference in timestamps in seconds (vectorized).
+        """
         return self.__op_expr__("timestamp_diff", other, inline=False, method=True)
 
     def date_diff(self, other):
-        return self.__op_expr__("date_diff", other, inline=False, method=True, check_types=False,)
+        """
+        Compute difference in dates in days (vectorized).
+        """
+        return self.__op_expr__(
+            "date_diff", other, inline=False, method=True, check_types=False,
+        )
 
     # noinspection PyPep8Naming
     def base_Sunday(self):
+        """
+        Compute prior Sunday date from date (self for Sundays) (vectorized).
+        """
         return self.__uop_expr__("base_Sunday")
 
 
@@ -693,9 +958,13 @@ def kop_expr(op, args, inline=False, method=False):
 
 
 class Value(Term):
+    """
+    Class for holding constants.
+    """
     def __init__(self, value):
         allowed = {
-            data_algebra.util.map_type_to_canonical(t) for t in [int, float, str, bool, type(None)]
+            data_algebra.util.map_type_to_canonical(t)
+            for t in [int, float, str, bool, type(None)]
         }
         disallowed = {data_algebra.util.map_type_to_canonical(type(value))} - allowed
         if len(disallowed) != 0:
@@ -709,22 +978,40 @@ class Value(Term):
         Term.__init__(self)
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         if not isinstance(other, Value):
             return False
         return self.value == other.value
 
     def get_views(self):
+        """
+        return list of unique views, expectation list is of size zero or one
+        """
         views = list()
         return views
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         return self
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         return self.value
 
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        """
+        Convert parsed expression into a string
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
+        """
         return PythonText(self.value.__repr__(), is_in_parens=False)
 
     # don't collect -5 as a complex expression
@@ -732,19 +1019,10 @@ class Value(Term):
         return Value(-self.value)
 
 
-class UnQuotedStr(str):
-    def __init__(self, v):
-        self.v = v
-        str.__init__(v)
-
-    def str(self):
-        return self.v
-
-    def __repr__(self):
-        return self.v
-
-
 class ListTerm(PreTerm):
+    """
+    Class to hold a collection.
+    """
     # derived from PreTerm as this is not combinable
     def __init__(self, value):
         assert isinstance(value, (list, tuple))
@@ -752,12 +1030,18 @@ class ListTerm(PreTerm):
         PreTerm.__init__(self)
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         if not isinstance(other, ListTerm):
             return False
         return self.value == other.value
 
     def get_views(self):
+        """
+        return list of unique views, expectation list is of size zero or one
+        """
         views = list()
         for ai in self.value:
             if isinstance(ai, PreTerm):
@@ -768,10 +1052,16 @@ class ListTerm(PreTerm):
         return views
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         new_list = [ai.replace_view(view) for ai in self.value]
         return ListTerm(new_list)
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         res = [None] * len(self.value)
         for i in range(len(self.value)):
             vi = self.value[i]
@@ -781,20 +1071,36 @@ class ListTerm(PreTerm):
         return res
 
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        """
+        Convert parsed expression into a string
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
+        """
         def li_to_python(value):
+            """convert a list item to Python"""
             try:
                 return str(value.to_python(want_inline_parens=False))
             except AttributeError:
                 return str(value)  # TODO: check if this should be repr?
 
-        return PythonText("[" + ", ".join([li_to_python(ai) for ai in self.value]) + "]", is_in_parens=False)
+        return PythonText(
+            "[" + ", ".join([li_to_python(ai) for ai in self.value]) + "]",
+            is_in_parens=False,
+        )
 
     def get_column_names(self, columns_seen):
+        """
+        Add column names to columns_seen
+        :param columns_seen: set of strings
+        :return:
+        """
         for ti in self.value:
             ti.get_column_names(columns_seen)
 
 
 class DictTerm(PreTerm):
+    """Class for carrying a dictionary or map."""
     # derived from PreTerm as this is not combinable
     # only holds values
     def __init__(self, value):
@@ -803,40 +1109,69 @@ class DictTerm(PreTerm):
         PreTerm.__init__(self)
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         if not isinstance(other, DictTerm):
             return False
         return self.value == other.value
 
     def get_views(self):
+        """
+        return list of unique views, expectation list is of size zero or one
+        """
         views = list()
         return views
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         return DictTerm(self.value)
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         return self.value.copy()
 
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        """
+        Convert parsed expression into a string
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
+        """
         def li_to_python(value):
+            """Convert an item to python"""
             try:
                 return str(value.to_python(want_inline_parens=False))
             except AttributeError:
                 return value.__repr__()
 
-        terms = [li_to_python(k) + ': ' + li_to_python(v) for k, v in self.value.items()]
+        terms = [
+            li_to_python(k) + ": " + li_to_python(v) for k, v in self.value.items()
+        ]
         return PythonText("{" + ", ".join(terms) + "}", is_in_parens=False)
 
     def get_column_names(self, columns_seen):
+        """
+        Add column names to columns_seen
+        :param columns_seen: set of strings
+        :return:
+        """
         pass
 
 
 def enc_value(value):
+    """
+    Encode a value as a PreTerm or derived class.
+    """
     if isinstance(value, PreTerm):
         return value
     if callable(value):
-        raise ValueError("callable as an argument")
+        raise ValueError("can't use a callable as an argument")
     if isinstance(value, list):
         return ListTerm(value)
     return Value(value)
@@ -863,9 +1198,15 @@ class ColumnReference(Term):
         Term.__init__(self)
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         return data_frame[self.column_name]
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         if not isinstance(other, ColumnReference):
             return False
@@ -874,17 +1215,34 @@ class ColumnReference(Term):
         return self.column_name == other.column_name
 
     def get_views(self):
+        """
+        return list of unique views, expectation list is of size zero or one
+        """
         views = list()
         views.append(self.view)
         return views
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         return ColumnReference(view=view, column_name=self.column_name)
 
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        """
+        Convert parsed expression into a string
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
+        """
         return PythonText(self.column_name, is_in_parens=False)
 
     def get_column_names(self, columns_seen):
+        """
+        Add column names to columns_seen
+        :param columns_seen: set of strings
+        :return:
+        """
         columns_seen.add(self.column_name)
 
 
@@ -938,6 +1296,9 @@ def _can_find_method_by_name(op):
 
 
 class Expression(Term):
+    """
+    Class for carrying an expression.
+    """
     def __init__(self, op, args, *, params=None, inline=False, method=False):
         assert isinstance(op, str)
         if not _can_find_method_by_name(op):
@@ -953,6 +1314,9 @@ class Expression(Term):
         Term.__init__(self)
 
     def is_equal(self, other):
+        """
+        Check if this expression code is the same as another expression.
+        """
         # can't use == as that builds a larger expression
         if not isinstance(other, Expression):
             return False
@@ -977,6 +1341,9 @@ class Expression(Term):
         return True
 
     def get_views(self):
+        """
+        return list of unique views, expectation list is of size zero or one
+        """
         views = list()
         for ai in self.args:
             vi = ai.get_views()
@@ -986,6 +1353,9 @@ class Expression(Term):
         return views
 
     def replace_view(self, view):
+        """
+        Move expression to a new view.
+        """
         new_args = [oi.replace_view(view) for oi in self.args]
         return Expression(
             op=self.op,
@@ -996,10 +1366,18 @@ class Expression(Term):
         )
 
     def get_column_names(self, columns_seen):
+        """
+        Add column names to columns_seen
+        :param columns_seen: set of strings
+        :return:
+        """
         for a in self.args:
             a.get_column_names(columns_seen)
 
     def evaluate(self, data_frame):
+        """
+        Evaluate expression, taking data from data_frame.
+        """
         args = [ai.evaluate(data_frame) for ai in self.args]
         # check user fns
         # first check chosen mappings
@@ -1032,6 +1410,12 @@ class Expression(Term):
         raise KeyError(f"function {self.op} not found")
 
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
+        """
+        Convert parsed expression into a string
+
+        :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
+        :return: PythonText
+        """
         n_args = len(self.args)
         if n_args <= 0:
             return PythonText(self.op + "()", is_in_parens=False)
@@ -1040,35 +1424,51 @@ class Expression(Term):
             if self.inline:
                 if sub_0.is_in_parens:
                     return PythonText(self.op + str(sub_0), is_in_parens=False)
-                return PythonText(self.op + '(' + str(sub_0) + ')', is_in_parens=False)
+                return PythonText(self.op + "(" + str(sub_0) + ")", is_in_parens=False)
             if self.method:
                 if sub_0.is_in_parens or isinstance(self.args[0], ColumnReference):
-                    return PythonText(str(sub_0) + "." + self.op + "()", is_in_parens=False)
-                return PythonText("(" + str(sub_0) + ")." + self.op + "()", is_in_parens=False)
+                    return PythonText(
+                        str(sub_0) + "." + self.op + "()", is_in_parens=False
+                    )
+                return PythonText(
+                    "(" + str(sub_0) + ")." + self.op + "()", is_in_parens=False
+                )
         if self.inline:
             subs = [str(ai.to_python(want_inline_parens=True)) for ai in self.args]
             result = (" " + self.op + " ").join(subs)
             if want_inline_parens:
-                return PythonText('(' + result + ')', is_in_parens=True)
+                return PythonText("(" + result + ")", is_in_parens=True)
             return PythonText(result, is_in_parens=False)
         subs = [ai.to_python(want_inline_parens=False) for ai in self.args]
         subs_0 = subs[0]
         subs = [str(si) for si in subs]
         if self.method:
             if subs_0.is_in_parens or isinstance(self.args[0], ColumnReference):
-                return PythonText(subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")", is_in_parens=False)
+                return PythonText(
+                    subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")",
+                    is_in_parens=False,
+                )
             else:
-                return PythonText("(" + subs[0] + ")." + self.op + "(" + ", ".join(subs[1:]) + ")", is_in_parens=False)
+                return PythonText(
+                    "(" + subs[0] + ")." + self.op + "(" + ", ".join(subs[1:]) + ")",
+                    is_in_parens=False,
+                )
         # treat as fn call
-        return PythonText(self.op + '(' + ", ".join(subs) + ')', is_in_parens=False)
+        return PythonText(self.op + "(" + ", ".join(subs) + ")", is_in_parens=False)
 
 
 # define with def so function has usable __name__
 def connected_components(f, g):
+    """
+    Compute connected components.
+    """
     return data_algebra.expr_rep.Expression(op="connected_components", args=[f, g])
 
 
 def standardize_join_type(join_str):
+    """
+    Replace join name with standard name.
+    """
     assert isinstance(join_str, str)
     join_str = join_str.upper()
     allowed = {"INNER", "LEFT", "RIGHT", "OUTER", "FULL", "CROSS"}
@@ -1079,6 +1479,9 @@ def standardize_join_type(join_str):
 
 # noinspection SpellCheckingInspection
 def get_columns_used(parsed_exprs):
+    """
+    Return set of columns used in an expression.
+    """
     assert isinstance(parsed_exprs, dict)
     columns_seen = set()
     for node in parsed_exprs.values():
@@ -1088,6 +1491,9 @@ def get_columns_used(parsed_exprs):
 
 # noinspection SpellCheckingInspection
 def implies_windowed(parsed_exprs):
+    """
+    Return true if expression implies a windowed calculation is needed.
+    """
     assert isinstance(parsed_exprs, dict)
     for opk in parsed_exprs.values():  # look for aggregation functions
         if isinstance(opk, data_algebra.expr_rep.Expression):
