@@ -1,5 +1,5 @@
 """
-Base clas for adapters for Pandas-like APIs
+Base class for adapters for Pandas-like APIs
 """
 
 from abc import ABC
@@ -108,13 +108,17 @@ def _type_safe_is_in(a, b):
 def _map_v(a, value_map, default_value):
     if len(value_map) > 0:
         type_a = data_algebra.util.guess_carried_scalar_type(a)
-        type_k = {data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.keys()}
+        type_k = {
+            data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.keys()
+        }
         if not data_algebra.util.compatible_types(type_k):
             raise TypeError(f"multiple types in dictionary keys: {type_k} in mapv()")
         type_k = list(type_k)[0]
         if not data_algebra.util.compatible_types([type_a, type_k]):
             raise TypeError(f"can't map {type_a} from a dict of {type_k}'s in mapv()")
-        type_v = {data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.values()}
+        type_v = {
+            data_algebra.util.map_type_to_canonical(type(v)) for v in value_map.values()
+        }
         if not data_algebra.util.compatible_types(type_v):
             raise TypeError(f"multiple types in dictionary values: {type_v} in mapv()")
         type_v = list(type_v)[0]
@@ -123,7 +127,7 @@ def _map_v(a, value_map, default_value):
             raise TypeError(f"default is {type_d} for {type_v} values in mapv()'s")
     # https://pandas.pydata.org/docs/reference/api/pandas.Series.map.html
     a = data_algebra.default_data_model.pd.Series(a)
-    a = a.map(value_map, na_action='ignore')
+    a = a.map(value_map, na_action="ignore")
     a[data_algebra.default_data_model.bad_column_positions(a)] = default_value
     return numpy.array(a.values)
 
@@ -202,7 +206,7 @@ def populate_impl_map(data_model):
             a, b
         ),
         "mapv": _map_v,
-        # fns that had been in bigquery_user_fns
+        # additonal fns
         # x is a pandas Series
         "as_int64": lambda x: x.astype("int64").copy(),
         "as_str": lambda x: x.astype("str").copy(),
@@ -259,6 +263,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     """
     Base class for implementing the data algebra on pandas-like APIs
     """
+
     def __init__(self, *, pd: types.ModuleType, presentation_model_name: str):
         assert isinstance(pd, types.ModuleType)
         data_algebra.data_model.DataModel.__init__(
@@ -270,6 +275,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     # utils
 
     def data_frame(self, arg=None):
+        """
+        Build a new emtpy data frame.
+        """
         if arg is None:
             # noinspection PyUnresolvedReferences
             return self.pd.DataFrame()
@@ -277,23 +285,38 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return self.pd.DataFrame(arg)
 
     def is_appropriate_data_instance(self, df):
+        """
+        Check if df is our type of data frame.
+        """
         # noinspection PyUnresolvedReferences
         return isinstance(df, self.pd.DataFrame)
 
     def can_convert_col_to_numeric(self, x):
+        """
+        Return True if column or value can be converted to numeric type.
+        """
         if isinstance(x, numbers.Number):
             return True
         # noinspection PyUnresolvedReferences
         return self.pd.api.types.is_numeric_dtype(x)
 
     def to_numeric(self, x, *, errors="coerce"):
+        """
+        Convert column to numeric.
+        """
         # noinspection PyUnresolvedReferences
         return self.pd.to_numeric(x, errors="coerce")
 
     def isnull(self, x):
+        """
+        Return vector indicating which entries are null (vectorized).
+        """
         return self.pd.isnull(x)
 
     def bad_column_positions(self, x):
+        """
+        Return vector indicating which entries are bad (null or nan) (vectorized).
+        """
         if self.can_convert_col_to_numeric(x):
             x = numpy.asarray(x + 0, dtype=float)
             return numpy.logical_or(
@@ -304,7 +327,10 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     # bigger stuff
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
-    def table_step(self, op, *, data_map, narrow):
+    def table_step(self, op, *, data_map: dict, narrow: bool):
+        """
+        Return data frame from table description and data_map.
+        """
         if op.node_name != "TableDescription":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.TableDescription"
@@ -312,12 +338,16 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         if (data_map is not None) and (len(data_map) > 0):
             df = data_map[op.table_name]
             if not self.is_appropriate_data_instance(df):
-                raise ValueError("data_map[" + op.table_name + "] was not the right type")
+                raise ValueError(
+                    "data_map[" + op.table_name + "] was not the right type"
+                )
         else:
             df = op.head
             assert df is not None
             if not self.is_appropriate_data_instance(df):
-                raise ValueError("Unnamed TableDescription stored was not the right type")
+                raise ValueError(
+                    "Unnamed TableDescription stored was not the right type"
+                )
         # check all columns we expect are present
         columns_using = op.column_names
         if not narrow:
@@ -352,6 +382,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
 
         # agg can return scalars, which then can't be made into a self.pd.DataFrame
         def promote_scalar(vi, *, target_len):
+            """
+            Convert a scalar into a vector.
+            """
             # noinspection PyBroadException
             try:
                 len_v = len(vi)
@@ -370,6 +403,12 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return self.pd.DataFrame(cols)
 
     def add_data_frame_columns_to_data_frame_(self, res, transient_new_frame):
+        """
+        Add columns from transient_new_frame to res. Res may be altered, and either of res or
+        transient_new_frame may be returned.
+        """
+        if transient_new_frame.shape[1] < 1:
+            return res
         if (res.shape[0] == 0) and (transient_new_frame.shape[0] > 0):
             # scalars get interpreted as single row items, instead of zero row items
             # growing the extension frame
@@ -393,6 +432,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def extend_step(self, op, *, data_map, narrow):
+        """
+        Execute an extend step, returning a data frame.
+        """
         if op.node_name != "ExtendNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ExtendNode")
         window_situation = (
@@ -505,6 +547,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def project_step(self, op, *, data_map, narrow):
+        """
+        Execute a project step, returning a data frame.
+        """
         if op.node_name != "ProjectNode":
             raise TypeError("op was supposed to be a data_algebra.data_ops.ProjectNode")
         # check these are forms we are prepared to work with, and build an aggregation dictionary
@@ -572,6 +617,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def select_rows_step(self, op, *, data_map, narrow):
+        """
+        Execute a select rows step, returning a data frame.
+        """
         if op.node_name != "SelectRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectRowsNode"
@@ -586,6 +634,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def select_columns_step(self, op, *, data_map, narrow):
+        """
+        Execute a select columns step, returning a data frame.
+        """
         if op.node_name != "SelectColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectColumnsNode"
@@ -596,6 +647,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res[op.column_selection]
 
     def drop_columns_step(self, op, *, data_map, narrow):
+        """
+        Execute a drop columns step, returning a data frame.
+        """
         if op.node_name != "DropColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.DropColumnsNode"
@@ -607,6 +661,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res[column_selection]
 
     def order_rows_step(self, op, *, data_map, narrow):
+        """
+        Execute an order rows step, returning a data frame.
+        """
         if op.node_name != "OrderRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.OrderRowsNode"
@@ -626,6 +683,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def rename_columns_step(self, op, *, data_map, narrow):
+        """
+        Execute a rename columns step, returning a data frame.
+        """
         if op.node_name != "RenameColumnsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.RenameColumnsNode"
@@ -637,6 +697,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
 
     # noinspection PyMethodMayBeStatic
     def standardize_join_code(self, jointype):
+        """
+        Map join names to Pandas names.
+        """
         assert isinstance(jointype, str)
         jointype = jointype.lower()
         mp = {
@@ -650,6 +713,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return jointype
 
     def natural_join_step(self, op, *, data_map, narrow):
+        """
+        Execute a natural join step, returning a data frame.
+        """
         if op.node_name != "NaturalJoinNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.NaturalJoinNode"
@@ -698,6 +764,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def concat_rows_step(self, op, *, data_map, narrow):
+        """
+        Execute a concat rows step, returning a data frame.
+        """
         if op.node_name != "ConcatRowsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConcatRowsNode"
@@ -730,6 +799,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         return res
 
     def convert_records_step(self, op, *, data_map, narrow):
+        """
+        Execute record conversion step, returning a data frame.
+        """
         if op.node_name != "ConvertRecordsNode":
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConvertRecordsNode"
